"""
Modern test suite for memhunt
"""
import pytest
from unittest.mock import Mock, patch

# Import the classes we want to test
from memhunt.browser.views import Start, RefCount, DebugView


class TestStart:
    """Test the Start view"""
    
    def test_start_view_instantiation(self):
        """Test that Start view can be instantiated"""
        context = Mock()
        request = Mock()
        view = Start(context, request)
        assert view is not None
        assert view.context == context
        assert view.request == request
    
    def test_start_view_call(self):
        """Test that Start view __call__ method works"""
        context = Mock()
        request = Mock()
        view = Start(context, request)
        
        # Mock the render_template method
        view.render_template = Mock(return_value="<html>test</html>")
        
        result = view()
        assert result == "<html>test</html>"
        view.render_template.assert_called_once()


class TestRefCount:
    """Test the RefCount view"""
    
    def setup_method(self):
        """Set up test fixtures"""
        self.context = Mock()
        self.request = Mock()
        self.request.form = {}
        self.request.response = Mock()
        self.view = RefCount(self.context, self.request)
    
    def test_refcount_view_instantiation(self):
        """Test that RefCount view can be instantiated"""
        assert self.view is not None
        assert self.view.context == self.context
        assert self.view.request == self.request
    
    def test_update_method(self):
        """Test the update method"""
        # Mock the render_template method to avoid template loading issues
        self.view.render_template = Mock()
        
        self.view.update()
        
        # Check that basic attributes are set
        assert hasattr(self.view, 'pairs')
        assert hasattr(self.view, 'total_ref_count')
        assert hasattr(self.view, 'garbage_containing')
        assert hasattr(self.view, 'garbage_watching')
        
        assert isinstance(self.view.pairs, list)
        assert isinstance(self.view.total_ref_count, int)
        assert isinstance(self.view.garbage_containing, int)
        assert isinstance(self.view.garbage_watching, int)
    
    def test_target_property_no_name(self):
        """Test target property when no name is provided"""
        self.request.form = {}
        result = self.view.target
        assert result is None
    
    def test_target_property_with_name(self):
        """Test target property with a name"""
        self.request.form = {'name': '<class \'str\'>'}
        # The target might be None if the class isn't found, which is expected
        result = self.view.target
        # Just check it doesn't crash
        assert result is None or isinstance(result, type)
    
    def test_view_backref_no_target(self):
        """Test view_backref when no target is selected"""
        self.view._target = None
        result = self.view.view_backref()
        assert result == "Please select an item to introspect"
    
    def test_view_ref_no_target(self):
        """Test view_ref when no target is selected"""
        self.view._target = None
        result = self.view.view_ref()
        assert result == "Please select an item to introspect"
    
    @patch('memhunt.browser.views.objgraph_lib.show_refs')
    def test_ref_file_with_target(self, mock_show_refs):
        """Test ref_file property with a valid target"""
        self.view._target = str  # Use str as a test target
        self.request.response.setHeader = Mock()
        
        # Mock the file operations
        with patch('tempfile.NamedTemporaryFile') as mock_temp:
            mock_file = Mock()
            mock_file.name = '/tmp/test.png'
            mock_temp.return_value.__enter__.return_value = mock_file
            
            with patch('builtins.open', create=True) as mock_open:
                mock_file_content = b'fake_image_data'
                mock_read = mock_open.return_value.__enter__.return_value.read
                mock_read.return_value = mock_file_content
                
                result = self.view.ref_file
                
                assert result == b'fake_image_data'
                expected_call = ('content-type', 'image/png')
                response_mock = self.request.response.setHeader
                response_mock.assert_called_with(*expected_call)


class TestDebugView:
    """Test the DebugView"""
    
    def setup_method(self):
        """Set up test fixtures"""
        self.context = Mock()
        self.request = Mock()
        self.view = DebugView(self.context, self.request)
    
    def test_debug_view_instantiation(self):
        """Test that DebugView can be instantiated"""
        assert self.view is not None
        assert self.view.context == self.context
        assert self.view.request == self.request
    
    @patch('memhunt.browser.views.objgraph_lib.most_common_types')
    def test_most_common(self, mock_most_common):
        """Test most_common method"""
        mock_most_common.return_value = [('dict', 100), ('list', 50)]
        
        # Mock the render_template method
        self.view.render_template = Mock(return_value="<html>common types</html>")
        
        result = self.view.most_common()
        
        assert result == "<html>common types</html>"
        assert len(self.view.pairs) == 2
        assert self.view.pairs[0]['name'] == 'dict'
        assert self.view.pairs[0]['refcount'] == 100
    
    def test_memory_method(self):
        """Test memory method"""
        result = self.view.memory()
        # Should return a string (formatted memory summary)
        assert isinstance(result, str)
        # Should not crash
        assert len(result) > 0
    
    def test_relative_memory_method(self):
        """Test relative_memory method"""
        result = self.view.relative_memory()
        # Should return a string (formatted memory summary)
        assert isinstance(result, str)
        # Should not crash
        assert len(result) > 0
    
    def test_reset_heap_method(self):
        """Test reset_heap method"""
        result = self.view.reset_heap()
        # Should return a string indicating reset is not supported
        assert isinstance(result, str)
        assert "reset" in result.lower()
    
    def test_by_referrers_method(self):
        """Test by_referrers method"""
        result = self.view.by_referrers()
        # Should return a string (formatted memory summary)
        assert isinstance(result, str)
        assert len(result) > 0
    
    def test_get_biggest_offender_method(self):
        """Test get_biggest_offender method"""
        result = self.view.get_biggest_offender()
        # Should return a string with biggest offender info
        assert isinstance(result, str)
        assert len(result) > 0
    
    def test_traverse_relative_memory_method(self):
        """Test traverse_relative_memory method"""
        result = self.view.traverse_relative_memory()
        # Should return a string (formatted memory summary)
        assert isinstance(result, str)
        assert len(result) > 0
    
    def test_breakpoint_method(self):
        """Test breakpoint method (should be disabled)"""
        result = self.view.breakpoint()
        # Should return security message
        assert isinstance(result, str)
        assert "security" in result.lower()
    
    def test_display_mem_method(self):
        """Test display_mem method"""
        result = self.view.display_mem()
        # Should return a string (either malloc stats or error message)
        assert isinstance(result, str)


class TestIntegration:
    """Integration tests"""
    
    def test_garbage_collection_interaction(self):
        """Test that the views work with garbage collection"""
        context = Mock()
        request = Mock()
        request.form = {}
        request.response = Mock()
        
        view = RefCount(context, request)
        view.template = Mock()
        
        # This should not crash even with real garbage collection
        view.update()
        
        assert isinstance(view.garbage_containing, int)
        assert isinstance(view.garbage_watching, int)
        assert view.garbage_containing >= 0
        assert view.garbage_watching >= 0
    
    def test_memory_tracking_basic(self):
        """Test basic memory tracking functionality"""
        context = Mock()
        request = Mock()
        
        debug_view = DebugView(context, request)
        
        # These should all return strings and not crash
        memory_result = debug_view.memory()
        relative_result = debug_view.relative_memory()
        biggest_result = debug_view.get_biggest_offender()
        
        assert isinstance(memory_result, str)
        assert isinstance(relative_result, str)
        assert isinstance(biggest_result, str)
        
        assert len(memory_result) > 0
        assert len(relative_result) > 0
        assert len(biggest_result) > 0


if __name__ == '__main__':
    pytest.main([__file__])


"""Pytest configuration for memhunt tests"""
import pytest
import sys
import os

# Add the package root to Python path
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))


@pytest.fixture
def mock_context():
    """Fixture providing a mock context object"""
    from unittest.mock import Mock
    return Mock()


@pytest.fixture
def mock_request():
    """Fixture providing a mock request object"""
    from unittest.mock import Mock
    request = Mock()
    request.form = {}
    request.response = Mock()
    return request


@pytest.fixture
def memory_view():
    """Fixture providing a DebugView instance for memory testing"""
    from memhunt.browser.views import DebugView
    from unittest.mock import Mock
    
    context = Mock()
    request = Mock()
    request.form = {}
    request.response = Mock()
    
    return DebugView(context, request)


@pytest.fixture
def ref_count_view():
    """Fixture providing a RefCount view instance for testing"""
    from memhunt.browser.views import RefCount
    from unittest.mock import Mock
    
    context = Mock()
    request = Mock()
    request.form = {}
    request.response = Mock()
    
    return RefCount(context, request)


"""
FastAPI integration example for memhunt

This module demonstrates how to integrate the memory debugging tools
with FastAPI applications.
"""
import logging
from typing import Dict, Any
from fastapi import FastAPI, HTTPException, Request
from fastapi.responses import HTMLResponse
from fastapi.templating import Jinja2Templates
import uvicorn

# Import our memory debugging tools
from pympler import muppy, summary
import objgraph

# Set up logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

app = FastAPI(title="Memory Hunt - FastAPI Edition",
              description="Memory debugging tools for FastAPI applications",
              version="0.2.0")

# Set up templates (you would need to create the template files)
templates = Jinja2Templates(directory="templates")


class MemoryDebugger:
    """Memory debugging utilities adapted for FastAPI"""
    
    def __init__(self):
        self.logger = logging.getLogger(self.__class__.__name__)
    
    def get_memory_summary(self) -> Dict[str, Any]:
        """Get a summary of current memory usage"""
        try:
            all_objects = muppy.get_objects()
            mem_summary = summary.summarize(all_objects)
            
            return {
                "total_objects": len(all_objects),
                "summary": [
                    {
                        "type": item[0],
                        "count": item[1],
                        "total_size": item[2]
                    }
                    for item in mem_summary[:20]  # Top 20 types
                ],
                "status": "success"
            }
        except Exception as e:
            self.logger.error(f"Error getting memory summary: {e}")
            return {"status": "error", "message": str(e)}
    
    def get_most_common_types(self) -> Dict[str, Any]:
        """Get most common object types using objgraph"""
        try:
            pairs = objgraph.most_common_types(limit=20)
            return {
                "most_common": [
                    {"type": pair[0], "count": pair[1]}
                    for pair in pairs
                ],
                "status": "success"
            }
        except Exception as e:
            self.logger.error(f"Error getting common types: {e}")
            return {"status": "error", "message": str(e)}
    
    def get_biggest_memory_users(self) -> Dict[str, Any]:
        """Get biggest memory consumers"""
        try:
            all_objects = muppy.get_objects()
            mem_summary = summary.summarize(all_objects)
            
            # Sort by total size
            sorted_summary = sorted(mem_summary, key=lambda x: x[2],
                                    reverse=True)
            
            return {
                "biggest_users": [
                    {
                        "type": item[0],
                        "count": item[1],
                        "total_size": item[2],
                        "avg_size": item[2] / item[1] if item[1] > 0 else 0
                    }
                    for item in sorted_summary[:10]  # Top 10
                ],
                "status": "success"
            }
        except Exception as e:
            self.logger.error(f"Error getting biggest users: {e}")
            return {"status": "error", "message": str(e)}


# Initialize the memory debugger
memory_debugger = MemoryDebugger()


@app.get("/", response_class=HTMLResponse)
async def root(request: Request):
    """Main page with links to debugging tools"""
    return templates.TemplateResponse("index.html", {
        "request": request,
        "title": "Memory Hunt - FastAPI Edition"
    })


@app.get("/api/memory/summary")
async def get_memory_summary() -> Dict[str, Any]:
    """Get memory usage summary"""
    return memory_debugger.get_memory_summary()


@app.get("/api/memory/common-types")
async def get_common_types() -> Dict[str, Any]:
    """Get most common object types"""
    return memory_debugger.get_most_common_types()


@app.get("/api/memory/biggest-users")
async def get_biggest_users() -> Dict[str, Any]:
    """Get biggest memory consumers"""
    return memory_debugger.get_biggest_memory_users()


@app.get("/api/memory/object-refs")
async def get_object_refs(
    object_type: str,
    max_depth: int = 3,
    format: str = "json"
):
    """
    Get reference graph for objects of a specific type
    
    Note: This would generate a graph file in a real implementation
    """
    try:
        # In a real implementation, you'd generate the graph
        # and return either the image or JSON representation
        return {
            "object_type": object_type,
            "max_depth": max_depth,
            "message": ("Reference graph generation not implemented "
                        "in this example"),
            "status": "info"
        }
    except Exception as e:
        logger.error(f"Error generating refs for {object_type}: {e}")
        raise HTTPException(status_code=500, detail=str(e))


@app.get("/debug/memory", response_class=HTMLResponse)
async def memory_debug_page(request: Request):
    """Memory debugging dashboard"""
    # Get all the memory data
    memory_summary = memory_debugger.get_memory_summary()
    common_types = memory_debugger.get_most_common_types()
    biggest_users = memory_debugger.get_biggest_memory_users()
    
    return templates.TemplateResponse("memory_debug.html", {
        "request": request,
        "memory_summary": memory_summary,
        "common_types": common_types,
        "biggest_users": biggest_users,
        "title": "Memory Debug Dashboard"
    })


@app.middleware("http")
async def log_requests(request: Request, call_next):
    """Middleware to log requests (useful for debugging)"""
    start_time = time.time()
    response = await call_next(request)
    process_time = time.time() - start_time
    
    logger.info(
        f"Path: {request.url.path} | "
        f"Method: {request.method} | "
        f"Status: {response.status_code} | "
        f"Process time: {process_time:.4f}s"
    )
    
    return response


if __name__ == "__main__":
    import time
    
    print("Starting Memory Hunt FastAPI server...")
    print("Visit http://localhost:8000 for the main page")
    print("Visit http://localhost:8000/docs for API documentation")
    print("Visit http://localhost:8000/debug/memory for memory debugging")
    
    uvicorn.run(
        "fastapi_example:app",
        host="127.0.0.1",
        port=8000,
        reload=True,
        log_level="info"
    )


"""
Modern test suite for memhunt

This replaces the old Plone-based tests with modern pytest-based tests.
For the actual test implementations, see the tests/ directory.
"""
import pytest


def test_suite():
    """
    Compatibility function for old test runners.
    Modern testing should use pytest directly.
    """
    # Run pytest programmatically if needed
    return pytest.main(['-v', 'tests/'])


if __name__ == '__main__':
    # Run the tests using pytest
    pytest.main(['-v', 'tests/', '--cov=objgraph'])




# -*- coding: utf-8 -*-
"""
This module contains the memhunt tool

"""
from setuptools import setup, find_packages


tests_require = [
    'pytest>=6.0',
    'pytest-cov>=2.10',
    'pytest-mock>=3.0',
]

setup(
      packages=find_packages(exclude=['ez_setup']),
      include_package_data=True,
      zip_safe=False,
      python_requires='>=3.8',
      install_requires=[
          'setuptools',
          'pympler>=0.9',
          'objgraph>=3.5.0',
          'jinja2>=3.0.0',
      ],
      tests_require=tests_require,
      extras_require={
          'tests': tests_require,
          'fastapi': [
              'fastapi>=0.68.0',
              'uvicorn>=0.15.0',
              'jinja2>=3.0.0',
          ],
          'dev': [
              'pytest>=6.0',
              'pytest-cov>=2.10',
              'pytest-mock>=3.0',
              'fastapi>=0.68.0',
              'uvicorn>=0.15.0',
          ],
      },
      test_suite='memhunt.tests.test_suite',
      )


import sys
import gc
import logging
from tempfile import NamedTemporaryFile
from typing import Optional, Union
from pathlib import Path

try:
    from jinja2 import Environment, FileSystemLoader, Template
except ImportError:
    # Fallback if Jinja2 is not available
    Environment = None
    FileSystemLoader = None
    Template = None

import objgraph as objgraph_lib
from pympler import muppy, summary

# Initialize pympler for memory tracking
memory_tracker = muppy
logger = logging.getLogger(__name__)


class BaseView:
    """Base view class for memory debugging tools"""
    
    def __init__(self, context=None, request=None):
        self.context = context
        self.request = request
        self._setup_templates()
    
    def _setup_templates(self):
        """Set up Jinja2 template environment"""
        if Environment is None:
            logger.warning("Jinja2 not available, templates will be disabled")
            self.jinja_env = None
            return
            
        # Get the directory containing the templates
        current_dir = Path(__file__).parent
        template_dir = current_dir / "templates"
        
        # Fallback to zpt directory if templates doesn't exist
        if not template_dir.exists():
            template_dir = current_dir / "zpt"
            
        if template_dir.exists():
            self.jinja_env = Environment(
                loader=FileSystemLoader(str(template_dir)),
                autoescape=True
            )
        else:
            logger.warning(f"Template directory not found: {template_dir}")
            self.jinja_env = None
    
    def render_template(self, template_name: str, **context) -> str:
        """Render a Jinja2 template with given context"""
        if self.jinja_env is None:
            return f"Template system not available. Template: {template_name}"
            
        try:
            template = self.jinja_env.get_template(template_name)
            context['view'] = self
            context['context'] = self.context
            context['request'] = self.request
            return template.render(**context)
        except Exception as e:
            logger.error(f"Error rendering template {template_name}: {e}")
            return f"Error rendering template: {e}"
    
    def render_simple_template(self, template_string: str, **context) -> str:
        """Render a simple template string"""
        if Template is None:
            return template_string
            
        try:
            template = Template(template_string)
            context['view'] = self
            return template.render(**context)
        except Exception as e:
            logger.error(f"Error rendering template string: {e}")
            return template_string


class Start(BaseView):
    """Start page view for memory debugging tools"""
    
    def __call__(self) -> str:
        title = "Memory and Reference Counting Tools"
        return self.render_template("start.html", title=title)


class RefCount(BaseView):
    """Reference counting view"""
    
    def __call__(self) -> str:
        self.update()
        return self.render_template("ref_count.html",
                                    pairs=self.pairs,
                                    total_ref_count=self.total_ref_count,
                                    garbage_containing=self.garbage_containing,
                                    garbage_watching=self.garbage_watching)
        
    def update(self) -> None:
        try:
            res = {}
            # collect all classes
            self.garbage_containing = len(gc.garbage)
            self.garbage_watching = len(gc.get_objects())
            self.total_ref_count = 0

            for mod in sys.modules.values():
                if mod is None:
                    continue
                for attr in dir(mod):
                    try:
                        obj = getattr(mod, attr)
                        if isinstance(obj, type):
                            res[obj] = sys.getrefcount(obj)
                    except (AttributeError, TypeError) as e:
                        logger.debug(f"Error accessing {attr}: {e}")
                        continue

            # sort by refcount
            pairs = [(repr(x[0]), x[1]) for x in res.items()]
            pairs.sort(key=lambda x: x[1], reverse=True)

            self.pairs = []
            for pair in pairs:
                self.total_ref_count += pair[1]
                self.pairs.append({'refcount': pair[1],
                                   'name': pair[0]})

            self.pairs = sorted(self.pairs, key=lambda x: x['refcount'])
            self.pairs.reverse()
        except Exception as e:
            logger.error(f"Error in update method: {e}")
            self.pairs = []
            self.total_ref_count = 0
            self.garbage_containing = 0
            self.garbage_watching = 0

    @property
    def target(self) -> Optional[type]:
        try:
            if getattr(self, '_target', None):
                return getattr(self, '_target')

            target = self.request.form.get('name', '')
            if not target:
                return None

            target = target.strip('<').strip('>')
            target = '_'.join(target.split(' ')[:-1])

            coll = {}
            for m in sys.modules.values():
                if m is None:
                    continue
                for sym in dir(m):
                    try:
                        o = getattr(m, sym)
                        if isinstance(o, type):
                            repr_str = repr(o).strip('<').strip('>')
                            name_parts = repr_str.split(' ')[:-1]
                            name = '_'.join(name_parts)
                            coll[name] = o
                    except (AttributeError, TypeError) as e:
                        logger.debug(f"Error processing symbol {sym}: {e}")
                        continue

            self._target = coll.get(target, None)
            return self._target
        except Exception as e:
            logger.error(f"Error in target property: {e}")
            return None

    def view_backref(self) -> Union[str, bytes]:
        if self.target is None:
            return "Please select an item to introspect"
        return self.back_ref_file

    def view_ref(self) -> Union[str, bytes]:
        if self.target is None:
            return "Please select an item to introspect"
        return self.ref_file

    @property
    def ref_file(self) -> bytes:
        try:
            self.request.response.setHeader('content-type', 'image/png')
            with NamedTemporaryFile('wb', suffix='.png', delete=False) as f:
                objgraph_lib.show_refs([self.target], max_depth=6, filename=f.name)
                with open(f.name, 'rb') as img_file:
                    return img_file.read()
        except Exception as e:
            logger.error(f"Error generating reference graph: {e}")
            return b"Error generating reference graph"

    @property
    def back_ref_file(self) -> bytes:
        try:
            self.request.response.setHeader('content-type', 'image/png')
            with NamedTemporaryFile('wb', suffix='.png', delete=False) as f:
                objgraph_lib.show_backrefs([self.target], max_depth=6, filename=f.name)
                with open(f.name, 'rb') as img_file:
                    return img_file.read()
        except Exception as e:
            logger.error(f"Error generating back-reference graph: {e}")
            return b"Error generating back-reference graph"

    def context_refs(self) -> bytes:
        self._target = self.context
        return self.ref_file

    def context_backrefs(self) -> bytes:
        self._target = self.context
        return self.back_ref_file


class DebugView(BaseView):
    """Debug view for memory analysis"""
    __allow_access_to_unprotected_subobjects__ = 1

    def most_common(self) -> str:
        pairs = objgraph_lib.most_common_types()
        self.pairs = []
        for pair in pairs:
            self.pairs.append({'refcount': pair[1],
                               'name': pair[0]})

        return self.render_template("ref_common_count.html", pairs=self.pairs)

    def display_mem(self) -> str:
        try:
            import malloc_stats
            return malloc_stats.malloc_stats()
        except ImportError:
            return "malloc_stats module not available"
        except Exception as e:
            logger.error(f"Error getting malloc stats: {e}")
            return f"Error getting malloc stats: {e}"

    def reset_heap(self) -> str:
        # Reset memory tracking baseline for testing
        # Note: pympler doesn't have exact equivalent to guppy's setrelheap
        return "Memory tracking reset (pympler doesn't support heap reset)"

    def memory(self) -> str:
        try:
            all_objects = muppy.get_objects()
            mem_summary = summary.summarize(all_objects)
            return '\n'.join(summary.format_(mem_summary))
        except Exception as e:
            logger.error(f"Error getting memory summary: {e}")
            return f"Error getting memory summary: {e}"

    # Print memory consumption summary
    def relative_memory(self) -> str:
        try:
            all_objects = muppy.get_objects()
            mem_summary = summary.summarize(all_objects)
            return '\n'.join(summary.format_(mem_summary))
        except Exception as e:
            logger.error(f"Error getting relative memory: {e}")
            return f"Error getting relative memory: {e}"

    def by_referrers(self) -> str:
        all_objects = muppy.get_objects()
        mem_summary = summary.summarize(all_objects)
        # Sort by size (closest equivalent to byrcs)
        sorted_summary = sorted(mem_summary, key=lambda x: x[2], reverse=True)
        return '\n'.join(summary.format_(sorted_summary))

    def get_biggest_offender(self) -> str:
        try:
            all_objects = muppy.get_objects()
            mem_summary = summary.summarize(all_objects)
            if mem_summary:
                biggest = max(mem_summary, key=lambda x: x[2])
                res = "BIGGEST OFFENDER:\\n"
                res += f"Type: {biggest[0]}\\n"
                res += f"Count: {biggest[1]}\\n"
                res += f"Total Size: {biggest[2]} bytes\\n"
                return res
            return "No memory data available"
        except Exception as e:
            logger.error(f"Error getting biggest offender: {e}")
            return f"Error getting biggest offender: {e}"

    # Print memory consumption with detailed analysis
    def traverse_relative_memory(self) -> str:
        try:
            all_objects = muppy.get_objects()
            mem_summary = summary.summarize(all_objects)
            # Get top 40 items (equivalent to get_rp(40))
            if len(mem_summary) >= 40:
                top_40 = mem_summary[:40]
            else:
                top_40 = mem_summary
            return '\n'.join(summary.format_(top_40))
        except Exception as e:
            logger.error(f"Error getting traverse relative memory: {e}")
            return f"Error getting traverse relative memory: {e}"

    def breakpoint(self) -> str:
        # Removed pdb.set_trace() for security reasons
        # Use proper logging or debugging tools instead
        return "Breakpoint functionality disabled for security"
